/*
 * Created on Aug 3, 2003
 */
package AcmeStudio.remote;

import java.rmi.RemoteException;

import AcmeStudio.remote.exception.SetupDesignException;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * Remote interface for remotely controlling AcmeStudio.<br>
 * <br>
 * Get a handle to an AcmeDesign by calling setupDesign; this handle provides
 * the access methods, via IAcmeStudioDesign, to manipulate the AcmeDesign.
 */
public interface IAcmeStudio extends IAcmeStudioRemoteControl {

    public boolean serverReady () throws RemoteException;

    /**
     * Sets up an AcmeDesign, by opening an editor for the file named by
     * <code>fileName</code> in project <code>projName</code>.
     * @param fileName  the path to the Acme file to open.
     * @param projName  the name of the project in which the Acme file resides
     * @return int  A unique ID identifying this AcmeDesign, required in all
     * other methods to operate on this AcmeDesign.  All subsequent calls to
     * this method will return the same ID.
     * @throws RemoteException
     * @throws SetupDesignException  Problem setting up design.
     */
    public IAcmeStudioDesign setupDesign (String fileName, String projName)
    throws RemoteException, SetupDesignException;

}
