/*
 * Created on Jul 5, 2005
 *  
 */
package AcmeStudio.remote.jms;

import java.io.IOException;
import java.io.PipedWriter;
import java.io.StringBufferInputStream;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.remote.exception.SetupDesignException;
import AcmeStudio.remote.impl.ASInfo;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeOpenedDesignRecord;
import acmeLib.elements.AcmeSystem;
/**
 * @author Bradley Schmerl Created Jul 5, 2005 An implementation of
 *                MessageListener that can subscribe to a JMS topic It listens for and
 *                process xAcme mutation events
 */
public class JMSListener implements MessageListener {

	private PipedWriter				_pipe;
	XAcmeToAcmeTranslator			_xAcmeTranslator;
	private static DocumentBuilder	docBuilder;
	static {
		try {
			docBuilder = DocumentBuilderFactory.newInstance ().newDocumentBuilder ();
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace ();
		}
		catch (FactoryConfigurationError e) {
			e.printStackTrace ();
		}
	}

	/**
	 * Called by JMS when a message is published on the subscribed topic
	 * 
	 * @param arg0
	 * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
	 */
	public void onMessage(Message msg) {
		// Parse message to XML
		String msgTxt = null;
		if (msg instanceof TextMessage) {
			TextMessage tmsg = (TextMessage )msg;
			try {
				msgTxt = tmsg.getText ();
			}
			catch (JMSException e) {
				e.printStackTrace ();
			}
		}
		if (msgTxt != null) {
			StringBufferInputStream msgIS = new StringBufferInputStream (msgTxt);
			pipe ("=>Received " + msgTxt);
			try {
				Document doc = docBuilder.parse (msgIS);
				Element msgXML = (Element )doc.getFirstChild ();
				_xAcmeTranslator.processXAcmeMutation (msgXML);
			}
			catch (Exception e) {
				// Ignore
			}
		}
	}

	/**
	 * @param msgXML
	 * @return
	 */
	private ASInfo setUpDesign(Element msgXML) {
		String projName = msgXML.getAttribute ("project");
		String fileName = msgXML.getAttribute ("file");
		IProject proj = null;
		if (projName != null) {
			pipe ("Received call to setup design for file '" + fileName + "' under project '" + projName + "'.");
			proj = AcmeStudioPlugin.getRoot ().getProject (projName);
		}
		if (proj == null) {
			pipe ("X> Unable to find project named " + projName + "!");
			proj = ASContext.current ().getActiveProject ();
			if (proj == null) { // fetch the first project in array
				pipe ("X> Unable to find currently active project!");
				proj = AcmeStudioPlugin.getRoot ().getProjects () [0];
				if (proj == null) {
					pipe ("X> Unable to find any project!!");
				}
			}
		}
		// find file
		final IFile file = (IFile )proj.findMember (fileName);
		if (file == null) {
			return null;
		}
		final ASInfo info = new ASInfo ();
		String absName = null;
		try {
			absName = file.getLocation ().toFile ().getCanonicalPath ();
		}
		catch (IOException e) {
			pipe ("X> " + e.toString ());
			e.printStackTrace ();
			absName = file.getLocation ().toString ();
		}
		info.setFileName (absName);
		pipe ("=> Opening editor on file '" + absName + "'");
		try {
			Display.getCurrent ().asyncExec (new Runnable () {

				public void run() {
					try {
						IWorkbenchPage page = AcmeStudioPlugin.getActiveWindow ().getActivePage ();
						info.setEditPart (page.openEditor (new org.eclipse.ui.part.FileEditorInput (file),
								"AcmeStudio.ui.editors.MultiPageAcmeEditor", OpenStrategy.activateOnOpen ()));
						// assume it's MPAE, get and store the SystemView
						info.setSystemView ((SystemView )info.mPAE ().getDiagramEditor ().getRootEditPart ().getFigure ());
						// get reference to AcmeSystem opened & store it
						AcmeOpenedDesignRecord rec = AcmeDesign.getDesignRecord (info.fileName ());
						if (rec != null) {
							AcmeDesign design = rec.getResolvedDesign ();
							if (design != null) {
								info.setDesign (design);
								AcmeSystem sys = (AcmeSystem )design.getSystems ().getNextItem ();
								info.setSystem (sys);
							}
						}
					}
					catch (PartInitException e) {
						throw new RuntimeException (
								new SetupDesignException (SetupDesignException.OPEN_FAILED, file.toString (), e));
					}
				}
			});
		}
		catch (RuntimeException e) {
			if (e.getCause () instanceof SetupDesignException) {
				pipe ("X> Open editor failed for file " + file.toString () + "!");
			}
			else {
				throw e;
			}
		}
		return info;
	}

	/**
	 * @param string
	 */
	private void pipe(String string) {
		try {
			_pipe.write (string + "\n");
		}
		catch (IOException e) {
			e.printStackTrace ();
		}
	}

	public PipedWriter getPipe() {
		if (_pipe == null) {
			_pipe = new PipedWriter ();
			_xAcmeTranslator = new XAcmeToAcmeTranslator (_pipe);
		}
		else {
			try {
				_pipe.flush ();
				_pipe.close ();
			}
			catch (IOException e) {
				// ignore
			}
			_pipe = new PipedWriter ();
			_xAcmeTranslator = new XAcmeToAcmeTranslator (_pipe);
		}
		return _pipe;
	}
}
