/*
 * Created on Jul 24, 2003
 */
package AcmeStudio.remote.test.statement;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.*;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * Shows the statements available to execute.
 */
public class HelpStatement extends Statement {

    public HelpStatement(String cmd) {
        super(cmd);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.test.Statement#execute(java.lang.Object)
     */
    public boolean execute(Object server) {
        boolean rv = true;

        if (params().length > 0) {  // assume only one param
            listRemoteCommands(Statement.parseParam((String) params()[0]));
        } else {
            // list local commands
            System.out.println("Local commands:");
            String[] cmds = StatementFactory.getCommands();
            for (int i=0; i < cmds.length; i++) {
                System.out.println("  " + StatementFactory.getDefaultStatement(cmds[i]).usage());
            }

            listRemoteCommands(server);
        }

        return rv;
    }

	/* (non-Javadoc)
     * @see AcmeStudio.remote.test.Statement#usage()
     */
    public String usage() {
        return cmd() + " [<cmd>]";
    }

    private void listRemoteCommands(Object server) {
        Class clazz = server.getClass();
        // list remote commands
        System.out.println("Remote commands for interface "
            + getShortName(clazz) + ":");
        // get and sort list of methods, as method signatures
        Method[] methods = clazz.getDeclaredMethods();
        List methodList = new ArrayList(methods.length);
        for (int i=0; i < methods.length; i++) {
            if (! Modifier.isPublic(methods[i].getModifiers())) continue;

            String name = methods[i].getName();
            // get return type
            String retType = getShortName(methods[i].getReturnType());
            // get list of param types for signature
            Class[] pTypes = methods[i].getParameterTypes();
            StringBuffer typeStr = new StringBuffer();
            if (pTypes.length > 0) {
                typeStr.append("<").append(getShortName(pTypes[0])).append(">");
                for (int j=1; j < pTypes.length; j++) {
                    typeStr.append(" <");
                    typeStr.append(getShortName(pTypes[j]));
                    typeStr.append(">");
                }
            }

            methodList.add(name + " " + typeStr.toString() + " : " + retType);
        }
        Collections.sort(methodList);
        for (Iterator i = methodList.iterator(); i.hasNext(); ) {
            // print method signature
            System.out.println("  " + i.next());
        }
    }

    private String getShortName (Class clazz) {
        if (clazz == null) {
            return "void";
        } else {
            String name = clazz.getName();
            int lIdx = name.lastIndexOf(".");
            return name.substring(lIdx + 1);
        }
    }

}
