/*
 * Created on Jul 24, 2003
 */
package AcmeStudio.remote.test.statement;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * Runs a script.
 */
public class RunScriptStatement extends Statement {

    private String _fileName = null;

    public RunScriptStatement(String cmd) {
        super(cmd);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.test.Statement#execute(java.lang.Object)
     */
    public boolean execute (Object server) {
        boolean rv = true;
        // run a script
        if (params().length < 1) {
            System.out.println("Usage: " + usage());
            rv = false;
        } else {
            // assume the only param is the file to load
            _fileName = (String) Statement.parseParam((String) params()[0]);
            List script = loadScript();
            Iterator i = script.iterator();
            while (i.hasNext()) {
                Statement stmt = (Statement) i.next();
                stmt.execute(server);
            }
            if (script.size() > 0) {  // we just went through the script
                System.out.println(">> Script '" + _fileName + "' done!");
            } else {
                System.out.println("X> Nothing to run in script '" + _fileName + "'!");
            }
        }
        return rv;
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.test.Statement#usage()
     */
    public String usage() {
        return cmd() + " <file to load>";
    }

    private List loadScript () {
        // find the file
        File file = new File(_fileName);
        if (! file.exists()) {  // try with extension
            System.out.println("Cannot find '" + _fileName + "'");
            int dotIdx = _fileName.lastIndexOf(".");
            String stub = _fileName;
            if (dotIdx > 0) {
                stub = _fileName.substring(0, dotIdx);
            }
            _fileName = stub + ".script";
            System.out.println("Trying '" + _fileName + "'...");
            file = new File(_fileName);
            if (file.exists()) {
                System.out.println("Found '" + _fileName + "'!");
            }
        }
        BufferedReader in = null;
        try {
            in = new BufferedReader(new FileReader(file));
        } catch (FileNotFoundException e) {
            System.err.println(e.getMessage());
            in = null;
        }

        List cmds = new ArrayList();
        if (in != null) {
            try {
                String line = null;
                do {
                    line = in.readLine();
                    if (line == null) break;
                    if (line.startsWith("#") || line.trim().equals("")) {
                        // skip it
                    } else {
                        Statement stmt = Statement.parse(line);
                        cmds.add(stmt);
                    }
                } while (line != null);
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    in.close();
                } catch (IOException e) { /* ignore */ }
            }
        }
        return cmds;
    }

}
