/*
 * Created on Jul 24, 2003
 */
package AcmeStudio.remote.test.statement;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * Factory to construct Statement objects of the given type. 
 */
public class StatementFactory {

    public static final String PACKAGE_PREFIX = "AcmeStudio.remote.test.statement.";

    public static final String CMD_CLEAR_VAR = "clvars";
    public static final String CMD_UNSET_VAR = "unset";
    public static final String CMD_PRINT_LINE = "println";
    public static final String CMD_GET_LIST_ITEM = "getitem";

    public static final String[] COMMANDS = {
        "run",
        "cwd",
        "set",
        CMD_UNSET_VAR,
        "var",
        "print",
        CMD_PRINT_LINE,
        "help",
        "vars",
        CMD_CLEAR_VAR,
        "pause",
        CMD_GET_LIST_ITEM
    };
    public static final String[] CLASSES = {
        PACKAGE_PREFIX + "RunScriptStatement",
        PACKAGE_PREFIX + "WorkingDirStatement",
        PACKAGE_PREFIX + "SetVarStatement",
        PACKAGE_PREFIX + "SetVarStatement",
        PACKAGE_PREFIX + "PrintVarStatement",
        PACKAGE_PREFIX + "PrintStatement",
        PACKAGE_PREFIX + "PrintStatement",
        PACKAGE_PREFIX + "HelpStatement",
        PACKAGE_PREFIX + "ListVarsStatement",
        PACKAGE_PREFIX + "ListVarsStatement",
        PACKAGE_PREFIX + "PauseStatement",
        PACKAGE_PREFIX + "ListAccessStatement"
    };

    public static Map _defaultStmt = new HashMap();

    public static Statement newStatement (String name) {
        Class clazz = null;
        try {
            clazz = Class.forName(name2Type(name));
            Constructor constructor = clazz.getConstructor(new Class[] {String.class});
            return (Statement) constructor.newInstance(new String[] {name});
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Returns whether the name supplied is a valid statement known to this
     * factory.
     * @param name  name of statement to query
     * @return boolean  Whether the name identifies a valid statement.
     */
    public static boolean isLocalStatement (String name) {
        return !name2Type(name).equals(name);
    }

    public static String[] getCommands () {
        return COMMANDS;
    }

    public static Statement getDefaultStatement (String name) {
        Statement retStmt = null;
        if (_defaultStmt.containsKey(name)) {
            retStmt = (Statement) _defaultStmt.get(name);
        } else {
            // generate a new one and store it
            retStmt = newStatement(name);
            if (retStmt != null) {
                _defaultStmt.put(name, retStmt);
            }
        }
        return retStmt;
    }

    private static String name2Type (String name) {
        String type = null;

        for (int i=0; i < COMMANDS.length; i++) {
            if (name.equalsIgnoreCase(COMMANDS[i])) {
                type = CLASSES[i];
                break;
            }
        }
        if (type == null) {  // non found, just use name as type
            type = name;
        }

        return type;
    }
}
