/*
 * Created on May 11, 2003
 *
 */
package AcmeStudio.remote.ui.views;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PipedReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import AcmeStudio.remote.AcmeStudioRemotePlugin;
import AcmeStudio.remote.RemoteControlSecurityManager;
import AcmeStudio.remote.impl.RemoteControlServer;
import AcmeStudio.remote.jms.JMSListener;
import AcmeStudio.remote.jms.XMLImporter;
import AcmeStudio.remote.utils.SetupRemote;

/**
 * Provides a view the shows the user what's going on with the AcmeStudio
 * Remote Control Server. 
 */
public class RemoteControlView extends ViewPart  {

    private Text _jmsCredentialsText;
    private Text _jmsPrincipalText;
    private Text _jmsProviderText;
    /** Resource bundle source */
    protected static final ResourceBundle BUNDLE = AcmeStudioRemotePlugin.getDefault().getResourceBundle();

    /** Amount of time in ms that the piper sleeps in between checking pipe */
    public static final int PIPER_SLEEP_TIME = 500;  //ms
    /** Name of the remote control server class */
    public static final String REMOTE_CONTROL = BUNDLE.getString("RemoteControlView.rmi.remoteControl.class"); //$NON-NLS-1$

    /** Label of the button indicating to Start remote server */
    protected static final String START_BUTTON = BUNDLE.getString("RemoteControlView.button.startControl");  //$NON-NLS-1$
    /** Label of the button indicating to Stop remote server */
    protected static final String STOP_BUTTON = BUNDLE.getString("RemoteControlView.button.stopControl");  //$NON-NLS-1$
    /** Label of the button indicating to connect to JMS **/
    protected static final String JMS_START_BUTTON = BUNDLE.getString("RemoteControlView.button.startJMS"); //$NON-NLS-1$
    /** Label of button indicating to stop JMS **/
    protected static final String JMS_STOP_BUTTOM = BUNDLE.getString("RemoteControlView.button.stopJMS"); //$NON-NLS-1$
    /**
     * This private Piper class monitors the pipe for content and updates text.
     */
    public class Piper extends Thread {
        private Thread _thread = null;
        private Display _display = null;

        /** Receiving end of communication channel with RemoteControlServer */
        private PipedReader _pipe = null;
        private BufferedReader _pipeBufReader = null;
		private final boolean	_useRMI;

        /**
         * Constructs a Piper object to monitor pipe.
         */
        public Piper(Display display, boolean useRMI) {
            super("Piper"); //$NON-NLS-1$
			_useRMI = useRMI;

            _thread = this;
            _display = display;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run()
         */
        public void run () {
            Thread currentThread = Thread.currentThread();
            while (_thread == currentThread) {
                try {
                    if (_pipe == null) {  // no pipe yet, so construct
                        initPipe();
                        sleep(PIPER_SLEEP_TIME);
                    } else {
                        synchronized (_pipe) {
                            _pipe.wait(PIPER_SLEEP_TIME);
                        }
                    }
                } catch (InterruptedException e) {
                    // ignore
                }

                // process pipe if pipe ready
                try {
                    if (_pipe != null && _pipeBufReader != null) {
                        final String line = _pipeBufReader.readLine();
                        _display.asyncExec(new Runnable() {
                            public void run() {
                                if (line.startsWith("=>")) { //$NON-NLS-1$
                                    appendText(line + "\n", _purpleColor, _whiteColor); //$NON-NLS-1$
                                } else  if (line.startsWith("X>")) { //$NON-NLS-1$
                                    appendText(line + "\n", _redColor, _whiteColor); //$NON-NLS-1$
                                } else {
                                    appendText(line + "\n"); //$NON-NLS-1$
                                }
                            }
                        });
                    }
                } catch (IOException e) {  // reconnect pipe
                    try {
                        _pipe.close();
                    } catch (IOException e2) { /* ignore */ }
                    try {
                        _pipeBufReader.close();
                    } catch (IOException e2) { /* ignore */ }
                    _pipe = null;;
                    _pipeBufReader = null;
                }
            }
        }
        
        public void terminate () {
            _thread = null;
        }
        
        private void initPipe () {
            _display.asyncExec(new Runnable() {
                public void run() {
                	if (_useRMI) {
	                    appendText("\n(Re)Connecting with RemoteControlServer pipe...", _greenColor, _whiteColor); //$NON-NLS-1$
	                    try {
	                        _pipe = new PipedReader(RemoteControlServer.getInstance().getPipe());
	                        _pipeBufReader = new BufferedReader(_pipe);
	                    } catch (IOException e) {
	                        appendText("[" + e.getMessage() + "]", _redColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
	                    }
	                    appendText("done!\n\n", _greenColor, _whiteColor); //$NON-NLS-1$
	                }
                	else { // use JMS
                		appendText ("\n(Re)Connecting with JMS Listener pipe...", _greenColor, _whiteColor);
                		try {
                			_pipe = new PipedReader (_jmsListener.getPipe ());
                			_pipeBufReader = new BufferedReader (_pipe);
                			appendText ("done\n\n", _greenColor, _whiteColor);
                		}
                		catch (IOException e) {
                			appendText("[" + e.getMessage() + "]", _redColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
                		}
                	}
                }
            });
        }
    }

    /** Colors */
    private Color _whiteColor = null;
    private Color _redColor = null;
    private Color _blueColor = null;
    private Color _greenColor = null;
    private Color _purpleColor = null;

    /** The Button widget to initiate remote control server **/
    private Button _start = null;
    /** The Button widget to initiate connection to JMS **/
    private Button _jms = null;
    /** The Label widget to indicate remote object hostname **/
    private Label _hostnameLabel = null;
    /** The Text widget to show remote object hostname for the remote control server **/
    private Text _hostnameText = null;
    /** The Label widget to indicate codebase **/
    private Label _codebaseLabel = null;
    /** The Text widget to show codebase of the classes for remote control server **/
    private Text _codebaseText = null;
    /** The Text widget to show the final codebase **/
    private Text _codebaseShow = null;
    /** The StyledText widget to show output **/
    private StyledText _text = null;
    /** The Text widget to show the JMS topic to listen to */
    private Text _jmsTopicText;
    /** Listener to respond to clicking of start button */
    private SelectionListener _startAdapter = new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
            if (_start == e.getSource()) {
                if (_start.getText().equals(START_BUTTON)) {
                    // start the remote control server
                    initRemoteControl();
                    _start.setText(STOP_BUTTON);
                } else {
                    // stop the remote control server
                    endRemoteControl();
                    _start.setText(START_BUTTON);
                }
            }
            else if (_jms ==  e.getSource ()) {
            	if (_jms.getText ().equals (JMS_START_BUTTON)) {
            		// start listening to JMS
            		startJMS ();
            		enableJMSOptions (false);
            		_jms.setText (JMS_STOP_BUTTOM);
            	}
            	else {
            		// stop listening to JMS
            		endJMS ();
            		_jms.setText (JMS_START_BUTTON);
            		enableJMSOptions (true);
            	}
            }
        }
    };
    /** Listener to respond to modification of hostname text */
    private ModifyListener _hostTextListener = new ModifyListener() {
        public void modifyText(ModifyEvent e) {
            if (_hostnameText == e.getSource()) {
                // update the code base text
                updateCodeBase();
            }
        }
    };
    /** Listener to respond to modification of codebase text */
    private ModifyListener _codebaseTextListener = new ModifyListener() {
        public void modifyText(ModifyEvent e) {
            if (_codebaseText == e.getSource()) {
                // update the code base text
                updateCodeBase();
            }
        }
    };
   

    /** Name of RMI server object */
    private String _rmiName = null;
    /** Codebase to use */
    private String _codebase = null;
    /** JMS topic to use */
	private String	_jmsTopic = null;
	
    /** An action to erase all the contents of the browser **/
    private Action _eraseContentsAction;

    /** Thread to monitor and process input from the pipe */
    private Piper _rmiPiper = null;
    private Piper _jmsPiper = null;
    
	private TopicConnection	_jmsConnection;
	private TopicSubscriber	_jmsSubscriber;
	private JMSListener	_jmsListener = new JMSListener ();


    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchPart#dispose()
     */
    public void dispose() {
        if (_whiteColor != null) {
            _whiteColor.dispose();
            _whiteColor = null;
        }
        if (_redColor != null) {
            _redColor.dispose();
            _redColor = null;
        }
        if (_blueColor != null) {
            _blueColor.dispose();
            _blueColor = null;
        }
        if (_greenColor != null) {
            _greenColor.dispose();
            _greenColor = null;
        }
        if (_purpleColor != null) {
            _purpleColor.dispose();
            _purpleColor = null;
        }
        if (_start != null && !_start.isDisposed()) {
            _start.removeSelectionListener(_startAdapter);
            _start = null;
        }
        if (_rmiPiper != null) {
            _rmiPiper.terminate();
            _rmiPiper = null;
        }
        if (_jmsPiper != null) {
        	_jmsPiper.terminate ();
        	_jmsPiper = null;
        }
        super.dispose();
    }

   
	/**
	 * @param b
	 */
	protected void enableJMSOptions(boolean enabled) {
		_jmsCredentialsText.setEnabled (enabled);
		_jmsPrincipalText.setEnabled (enabled);
		_jmsProviderText.setEnabled (enabled);
		_jmsTopicText.setEnabled (enabled);
	}


	/**
	 * 
	 */
	protected void endJMS() {
		appendText ("Closing connection to JMS...", _greenColor, _whiteColor);
		try {
			_jmsSubscriber.setMessageListener(null);
			_jmsConnection.close ();
			appendText ("done\n", _greenColor, _whiteColor);
		}
		catch (Exception e) {
			appendText ("\nFailed to close connection to JMS: " + e.getMessage (), _redColor, _whiteColor);
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	protected void startJMS() {
//		ClassLoader loader = AcmeStudioRemotePlugin.getDefault ().getClass ().getClassLoader ();
//		Thread.currentThread().setContextClassLoader(loader);
		Class clazz = XMLImporter.class;
		System.setSecurityManager (new RemoteControlSecurityManager ());
		final Hashtable env = new Hashtable ();
		env.put (Context.INITIAL_CONTEXT_FACTORY, BUNDLE.getString ("RemoteControlView.jms.contextFactory")); //$NON-NLS-1$
		env.put (Context.PROVIDER_URL, _jmsProvider);
		env.put (Context.SECURITY_PRINCIPAL, _jmsPrincipal);
		env.put (Context.SECURITY_CREDENTIALS, _jmsCredentials);
		appendText ("Connecting to JMS...", _greenColor, _whiteColor);
		try {
			// Make sure the class loader finds the class
			// Got this from: http://dev.eclipse.org/newslists/news.eclipse.platform/msg25571.html
			class InitialContextGetter implements Runnable {

				InitialContext	result;

				public void run() {
					try {
						result = new InitialContext (env);
					}
					catch (NamingException e) {
						result = null;
					}
				}
			}
			ClassLoader loader = AcmeStudioRemotePlugin.getDefault ().getClass ().getClassLoader ();
			InitialContextGetter getter = new InitialContextGetter ();
			Thread getterThread = new Thread (getter);
			getterThread.setContextClassLoader (loader);
			getterThread.start ();
			try {
				getterThread.join ();
			}
			catch (Exception aEx) {
				aEx.printStackTrace ();
			}
			Context ctx = getter.result;
			//				Context ctx = new InitialContext (env);
			TopicConnectionFactory factory = (TopicConnectionFactory )ctx.lookup (BUNDLE
					.getString ("RemoteControlView.jms.topicConnectionFactory")); //$NON-NLS-1$
			Topic acmeRemoteTopic = (Topic )ctx.lookup ("jms/AcmeStudioRemote"); //$NON-NLS-1$
			_jmsConnection = factory.createTopicConnection ();
			TopicSession session = _jmsConnection.createTopicSession (false, Session.AUTO_ACKNOWLEDGE);
			_jmsSubscriber = session.createSubscriber (acmeRemoteTopic);
			_jmsSubscriber.setMessageListener (_jmsListener);
			_jmsConnection.start ();
			appendText ("done\n", _greenColor, _whiteColor);
		}
		catch (NamingException e) {
			appendText ("\nFailed to connect to JMS: " + e.getMessage (), _redColor, _whiteColor);
			e.printStackTrace ();
		}
		catch (JMSException e) {
			appendText ("\nFailed to connect to JMS: " + e.getMessage (), _redColor, _whiteColor);
			e.printStackTrace ();
		}
		if (_jmsPiper == null) {
			_jmsPiper = new Piper (getSite ().getShell ().getDisplay (), false);
			_jmsPiper.start ();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite)
	 */
    public void init(IViewSite site) throws PartInitException {
        super.init(site);

        _whiteColor = new Color(getSite().getShell().getDisplay(), 255, 255, 255);
        _redColor = new Color(getSite().getShell().getDisplay(), 255, 0, 0);
        _blueColor = new Color(getSite().getShell().getDisplay(), 0, 50, 255);
        _greenColor = new Color(getSite().getShell().getDisplay(), 0, 200, 40);
        _purpleColor = new Color(getSite().getShell().getDisplay(), 200, 0, 200);
    }
    
    private static final int COLS = 5;
	protected String	_jmsProvider;
	protected String	_jmsPrincipal;
	protected String	_jmsCredentials;

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    public void createPartControl(Composite p) {
    	
        String hostnameProp = null;
        try {
            hostnameProp = BUNDLE.getString("RemoteControlView.text.hostname"); //$NON-NLS-1$
        } catch (MissingResourceException e) {
            hostnameProp = BUNDLE.getString("Plugin.host"); //$NON-NLS-1$
        }
        String codebaseProp = null;
        try {
            codebaseProp = BUNDLE.getString("RemoteControlView.text.codebase"); //$NON-NLS-1$
        } catch (MissingResourceException e) {
            codebaseProp = AcmeStudioRemotePlugin.getPluginPath(BUNDLE.getString("RemoteControlView.rmi.path.remote")); //$NON-NLS-1$
        }
        String jmsTopicProp = BUNDLE.getString ("Plugin.jmsTopic"); //$NON-NLS-1$
        String jmsProviderProp = BUNDLE.getString ("RemoteControlView.jms.providerURL"); //$NON-NLS-1$
        String jmsPrincipalProp = BUNDLE.getString ("RemoteControlView.jms.principal"); //$NON-NLS-1$
        String jmsCredentialProp = BUNDLE.getString ("RemoteControlView.jms.credentials"); //$NON-NLS-1$
        
    	p.setLayout (new FillLayout (SWT.VERTICAL));
    	Composite parent = new Composite (p, SWT.NONE);
    	parent.setLayout (new FillLayout (SWT.VERTICAL));

    	TabFolder optionsTab = new TabFolder(parent, SWT.NONE);

    	TabItem rmiTab = new TabItem(optionsTab, SWT.NONE);
    	rmiTab.setText("RMI"); //$NON-NLS-1$

    	Composite rmiParent = new Composite (optionsTab, SWT.NONE);
        GridLayout gl = new GridLayout(COLS , false);
        rmiParent.setLayout(gl);
    	rmiTab.setControl(rmiParent);

        _hostnameLabel = new Label(rmiParent, SWT.RIGHT);
        GridData hlgd = new GridData();
        hlgd.horizontalAlignment = GridData.END;
        _hostnameLabel.setLayoutData(hlgd);
        _hostnameLabel.setText(BUNDLE.getString("RemoteControlView.label.hostname")); //$NON-NLS-1$

        _hostnameText = new Text(rmiParent, SWT.SINGLE | SWT.BORDER);
        GridData htgd = new GridData();
        htgd.horizontalAlignment = GridData.BEGINNING;
        htgd.widthHint = 150;
        htgd.heightHint = 15;
        _hostnameText.setLayoutData(htgd);
        _hostnameText.setText(hostnameProp);
        _hostnameText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.hostname")); //$NON-NLS-1$
        _hostnameText.setBackground(_whiteColor);
        _hostnameText.addModifyListener(_hostTextListener);

        _codebaseLabel = new Label(rmiParent, SWT.RIGHT);
        GridData clgd = new GridData();
        clgd.horizontalAlignment = GridData.END;
        _codebaseLabel.setLayoutData(clgd);
        _codebaseLabel.setText(BUNDLE.getString("RemoteControlView.label.codebase")); //$NON-NLS-1$

        _codebaseText = new Text(rmiParent, SWT.SINGLE | SWT.BORDER);
        GridData ctgd = new GridData();
        ctgd.horizontalAlignment = GridData.FILL;
        ctgd.grabExcessHorizontalSpace = true;
        ctgd.heightHint = 15;
        ctgd.horizontalSpan = 2;
        _codebaseText.setLayoutData(ctgd);
        _codebaseText.setText(codebaseProp);
        _codebaseText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.rmiPath")); //$NON-NLS-1$
        _codebaseText.setBackground(_whiteColor);
        _codebaseText.addModifyListener(_codebaseTextListener);

        _codebaseShow = new Text(rmiParent, SWT.MULTI | SWT.WRAP | SWT.READ_ONLY);
        GridData csgd = new GridData();
        csgd.horizontalAlignment = GridData.FILL;
        csgd.grabExcessHorizontalSpace = true;
        csgd.heightHint = 45;
        csgd.horizontalSpan = COLS-1;
        _codebaseShow.setLayoutData(csgd);
        _codebaseShow.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.codebaseFinal")); //$NON-NLS-1$
        _codebaseShow.setForeground(_blueColor);


 
        
        _start = new Button(rmiParent, SWT.CENTER);
        _start.setText(START_BUTTON);
        _start.addSelectionListener(_startAdapter);
    	_start.addSelectionListener (_startAdapter);

    	
        GridData rgd = new GridData();
        rgd.horizontalAlignment = GridData.END;
        rgd.horizontalSpan = 1;
        _start.setLayoutData(rgd);

    	TabItem jmsTab = new TabItem(optionsTab, SWT.NONE);
    	jmsTab.setText("JMS"); //$NON-NLS-1$

    	Composite jmsParent = new Composite(optionsTab, SWT.NONE);
    	GridLayout gridLayout = new GridLayout();
    	gridLayout.numColumns = 4;
    	jmsParent.setLayout(gridLayout);
    	jmsTab.setControl(jmsParent);

    	Label label = new Label(jmsParent, SWT.NONE);
    	label.setText("JMS Provider:"); //$NON-NLS-1$

    	_jmsProviderText = new Text(jmsParent, SWT.BORDER);
    	_jmsProviderText.addModifyListener(new ModifyListener() {
    		public void modifyText(ModifyEvent e) {
    			_jmsProvider = _jmsProviderText.getText();
    		}
    	});
    	_jmsProviderText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	_jmsProviderText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.jmsProvider")); //$NON-NLS-1$
    	_jmsProviderText.setText(BUNDLE.getString("RemoteControlView.jms.providerURL")); //$NON-NLS-1$

    	label = new Label(jmsParent, SWT.NONE);
    	label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
    	label.setText("JMS Topic:"); //$NON-NLS-1$

    	_jmsTopicText = new Text(jmsParent, SWT.BORDER);
    	_jmsTopicText.addModifyListener(new ModifyListener() {
    		public void modifyText(ModifyEvent e) {
    			_jmsTopic = _jmsTopicText.getText ();
    		}
    	});
    	_jmsTopicText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.jmsTopic")); //$NON-NLS-1$
    	GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    	_jmsTopicText.setLayoutData(gridData);
    	_jmsTopicText.setText (jmsTopicProp);

    	label = new Label(jmsParent, SWT.NONE);
    	label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
    	label.setText("Principal:"); //$NON-NLS-1$

    	_jmsPrincipalText = new Text(jmsParent, SWT.BORDER);
    	_jmsPrincipalText.addModifyListener(new ModifyListener() {
    		public void modifyText(ModifyEvent e) {
    			_jmsPrincipal = _jmsPrincipalText.getText ();
    		}
    	});
    	_jmsPrincipalText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
    	_jmsPrincipalText.setText (BUNDLE.getString ("RemoteControlView.jms.principal")); //$NON-NLS-1$
    	_jmsPrincipalText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.jmsPrincipal")); //$NON-NLS-1$
    	
    	label = new Label(jmsParent, SWT.NONE);
    	label.setText("Credentials:"); //$NON-NLS-1$

    	_jmsCredentialsText = new Text(jmsParent, SWT.PASSWORD | SWT.BORDER);
    	_jmsCredentialsText.addModifyListener(new ModifyListener() {
    		public void modifyText(ModifyEvent e) {
    			_jmsCredentials = _jmsCredentialsText.getText ();
    		}
    	});
    	_jmsCredentialsText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
    	_jmsCredentialsText.setText (BUNDLE.getString ("RemoteControlView.jms.credentials")); //$NON-NLS-1$
    	_jmsCredentialsText.setToolTipText(BUNDLE.getString("RemoteControlView.text.tooltip.jmsCredentials")); //$NON-NLS-1$
    	
    	_jms = new Button(jmsParent, SWT.NONE);
    	gridData = new GridData(GridData.FILL_HORIZONTAL);
    	gridData.horizontalSpan = 4;
    	_jms.setLayoutData(gridData);
    	_jms.setText (JMS_START_BUTTON);
    	_jms.addSelectionListener(_startAdapter);
    

        _text = new StyledText(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
        _text.setBackground(_whiteColor);
        _text.setForeground(_blueColor);
       
        updateCodeBase();

//        makeActions ();
//        fillActions ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {

    }

    /**
     * Appends the supplied string to text using default colors for foreground
     * and background.
     * @param str  the text string to append
     */
    public void appendText (String str) {
        _text.append(str);
        _text.setSelection(_text.getCharCount());
    }

    /**
     * Appends the supplied string to text and change that portion of the text
     * to the supplied color.
     * @param str  the text string to append
     * @param fg  the color to style the text
     * @param bg  the color to style the background
     */
    public void appendText (String str, Color fg, Color bg) {
        int start = _text.getCharCount();
        int len = str.length();

        appendText(str);

        // change color of text
        StyleRange[] styles = new StyleRange[1];
        styles[0] = new StyleRange(start, len, fg, bg);
        _text.replaceStyleRanges(start, len, styles);
        _text.redrawRange(start, len, true);
    }

    /**
     * Takes the text from the hostname text widget and that from the codebase
     * text widget, and compose the new codebase that would be set for the
     * remote control server object.
     */
    private void updateCodeBase () {
        String hostname = _hostnameText.getText();
        String codebase = _codebaseText.getText();
        // replace all spaces
        codebase = codebase.replaceAll("[ ]", "%20"); //$NON-NLS-1$ //$NON-NLS-2$
        
        URL url = null;
        try {
            // attempt to construct a new url using the given codebase
            url = new URL(codebase);
            codebase = url.toString();
        } catch (MalformedURLException e) {
            // try as local file
            try {
                url = new URL("file", null, codebase); //$NON-NLS-1$
                codebase = url.toString();
            } catch (MalformedURLException e2) {
                codebase = null;
            }
        }

        _codebase = codebase;
        _codebaseShow.setText("Remote object sits on " + hostname //$NON-NLS-1$
            + "\nRMI codebase is "+ codebase); //$NON-NLS-1$
    }

    
    /**
	 * Takes the text from the topic text widget and sets the new topic to that 
	 */
	protected void updateTopic() {
		String topic = _jmsTopicText.getText ();
		if (!topic.startsWith ("jms/")) { //$NON-NLS-1$
			topic = "jms/" + topic; //$NON-NLS-1$
		}
		_jmsTopic = topic;
	}

    private boolean _registryStarted = false;
    private void initRegistry () {
		try {
			// start the rmi registry
            appendText("Starting RMI Registry...", _greenColor, _whiteColor); //$NON-NLS-1$
			SetupRemote.startRegistry();
            _registryStarted = true;
			appendText("done!\n", _greenColor, _whiteColor); //$NON-NLS-1$
		} catch (RemoteException e) {
			appendText("\nFailed to start RMI Registry:  " + e.toString() + "\n\n", _redColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
		}
    }

    private void initRemoteControl () {
        if (! _registryStarted) {
            initRegistry();
        }

        try {
            appendText("Setting up " + REMOTE_CONTROL + " object...", _greenColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
            _rmiName = SetupRemote.setupUnicast(_codebase, _hostnameText.getText(), REMOTE_CONTROL);
            appendText("done!\n", _greenColor, _whiteColor); //$NON-NLS-1$
        } catch (Exception e) {
            appendText("\nFailed to initialize remote control:  " + e.toString() + "\n\n", _redColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
        }

        if (_rmiPiper == null) {
            _rmiPiper = new Piper(getSite().getShell().getDisplay(), true);
            _rmiPiper.start();
        }
    }

    private void endRemoteControl () {
        if (_rmiName == null) {
            appendText("\nNothing to terminate.\n\n", _greenColor, _whiteColor); //$NON-NLS-1$
            return;
        }

        try {
            appendText("\nTerminating " + REMOTE_CONTROL + " object...", _greenColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
            SetupRemote.teardownUnicast(_rmiName);
            appendText("done!\n\n", _greenColor, _whiteColor); //$NON-NLS-1$
        } catch (Exception e) {
            appendText("\nFailed to terminate remote control:  " + e.toString() + "\n\n", _redColor, _whiteColor); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * Puts all the actions in the tool bar manager
     */
    private void fillActions() {
//        IToolBarManager mgr = getViewSite ().getActionBars().getToolBarManager();
//        mgr.add (_eraseContentsAction);
    }

    /**
     * Make all the actions associated with this browser
     */
    private void makeActions() {
//        _eraseContentsAction = new Action () {
//            public void run () {
//                _rootTypes.clear();
//                _viewer.refresh();   
//            }   
//        };
//        _eraseContentsAction.setText ("Clear view");
//        _eraseContentsAction.setToolTipText("Clears all the types from the view");
//        _eraseContentsAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_CLEAR));
    }
	private void createActions() {
	}
	private void initializeToolBar() {
		IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
	}
	private void initializeMenu() {
		IMenuManager menuManager = getViewSite().getActionBars().getMenuManager();
	}

}
