/*
 * Created on Jul 17, 2003
 * 
 * Adapted from Java Tutorial URL:
 * http://java.sun.com/j2se/1.3/docs/guide/rmi/activation/code/Setup.java.
 * 
 * Sun license appears below:
 */
/*
 * Copyright (c) 1998 - 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */
package AcmeStudio.remote.utils;

import java.io.FileInputStream;
import java.rmi.MarshalledObject;
import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.activation.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Properties;

//import org.apache.log4j.Logger;
//import org.apache.log4j.PropertyConfigurator;
import org.eclipse.core.runtime.IPath;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.remote.AcmeStudioRemotePlugin;
import AcmeStudio.remote.RemoteControlSecurityManager;
import AcmeStudio.remote.rmi.RemoteControlUnicast;

/**
 * This class registers information about the ActivatableImplementation class
 * with rmid and the rmiregistry
 */
public class SetupRemote {

    /** RMI URI path separator */
    public static final String SEP = AcmeStudioRemotePlugin.getResourceString("Plugin.file.separator");
    /** RMI URI path separator */
    public static final String PORT_SEP = AcmeStudioRemotePlugin.getResourceString("Plugin.port.separator");
    /** Host name */
    public static final String HOST = AcmeStudioRemotePlugin.getResourceString("Plugin.host");
    /** RMI port */
    public static final int RMI_PORT = Registry.REGISTRY_PORT;
    /** Logger configuration file */
    public static final String LOG_CONF_FILE = AcmeStudioRemotePlugin.getResourceString("Plugin.logger.config");
    /** Activation Group security policy file */
    public static final String GROUP_SECURITY_POLICY = AcmeStudioRemotePlugin.getResourceString("Plugin.group.security.policy");

	public static void startRegistry () throws RemoteException {
    	LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
    	try {
			Thread.sleep(2000);  // make sure registry starts!
		} catch (InterruptedException e) {  // ignore
		}
	}

    public static String setupUnicast (String codebase, String hostname, String remoteObj) throws Exception {

        SecurityManager sm = System.getSecurityManager();
        if (sm == null) {
            System.setSecurityManager(new RemoteControlSecurityManager());
        }

        if (System.getProperty("java.rmi.server.codebase") == null) {
            System.setProperty("java.rmi.server.codebase", codebase);
        }
        System.err.println("** java.rmi.server.codebase set to '" + System.getProperty("java.rmi.server.codebase") + "'");
        if (System.getProperty("java.rmi.server.hostname") == null) {
            System.setProperty("java.rmi.server.hostname", hostname);
        }
        System.err.println("** java.rmi.server.hostname set to '" + System.getProperty("java.rmi.server.hostname") + "'");
        System.setProperty("java.rmi.server.useCodebaseOnly", "true");

//        Logger log = null;
        // Set up logging for the activatable class
        Properties logProps = new Properties();
        logProps.load(new FileInputStream(AcmeStudioRemotePlugin.getPluginPath(LOG_CONF_FILE)));
        // get the right file name
        IPath path = AcmeStudioPlugin.getDefault().getStateLocation();
        path = path.append(AcmeStudioRemotePlugin.getResourceString("Plugin.logger.debugfile.value"));
        logProps.setProperty(AcmeStudioRemotePlugin.getResourceString("Plugin.logger.debugfile.property"),
            path.toString());
//        PropertyConfigurator.configure(logProps);
//        log = Logger.getLogger(activatable);

        // Instantiate a remote object
        Remote remote = new RemoteControlUnicast();
//        if (log != null) log.info("* Got the stub for " + activatable);

        // Bind the stub to a name in the registry running on 1099
        String rmiName = SEP + SEP + HOST + PORT_SEP + RMI_PORT + SEP + remoteObj;
        Naming.rebind(rmiName, remote);
//        if (log != null) log.info("* Exported " + rmiName);

        return rmiName;
    }

    public static void teardownUnicast (String remoteObj) throws Exception {

        // unset security manager and properties
        System.setSecurityManager(null);
        Properties props = System.getProperties();
        props.remove("java.rmi.server.codebase");
        props.remove("java.rmi.server.hostname");
        props.remove("java.rmi.server.useCodebaseOnly");

        Naming.unbind(remoteObj);
    }

	public static void setupActivatable (String location, String activatable) throws Exception {

		System.setSecurityManager(new RemoteControlSecurityManager());

//		  Logger log = null;
		// Set up logging for the activatable class
		Properties logProps = new Properties();
		logProps.load(new FileInputStream(AcmeStudioRemotePlugin.getPluginPath(LOG_CONF_FILE)));
		// get the right file name
		IPath path = AcmeStudioPlugin.getDefault().getStateLocation();
		path = path.append(AcmeStudioRemotePlugin.getResourceString("Plugin.logger.debugfile.value"));
		logProps.setProperty(AcmeStudioRemotePlugin.getResourceString("Plugin.logger.debugfile.property"),
			path.toString());
//		  PropertyConfigurator.configure(logProps);
//		  log = Logger.getLogger(activatable);

		// Because of the 1.2 security model, a security policy should 
		// be specified for the ActivationGroup VM.
		Properties props = new Properties();
		props.put("java.security.policy", AcmeStudioRemotePlugin.getPluginPath(GROUP_SECURITY_POLICY));

		ActivationGroupDesc.CommandEnvironment ace = null;
		ActivationGroupDesc transGrp = new ActivationGroupDesc(props, ace);

		// Once the ActivationGroupDesc has been created, register it 
		// with the activation system to obtain its ID
		ActivationGroupID agi = ActivationGroup.getSystem().registerGroup(transGrp);

		// Create the rest of the parameters that will be passed to
		// the ActivationDesc constructor
		MarshalledObject data = null;

		// The location argument to the ActivationDesc constructor will be used 
		// to uniquely identify this class; it's location is relative to the  
		// URL-formatted String, location.
		ActivationDesc desc =
			new ActivationDesc(agi, activatable, location, data);

		// Register with rmid
		Remote remote = Activatable.register(desc);
//		  if (log != null) log.info("* Got the stub for " + activatable);

		// Bind the stub to a name in the registry running on 1099
		String rmiName = SEP + SEP + HOST + PORT_SEP + RMI_PORT + SEP + activatable;
		Naming.rebind(rmiName, remote);
//		  if (log != null) log.info("* Exported " + rmiName);
	}
}
