/*
 * Created on Jul 31, 2003
 */
package AcmeStudio.remote.exception;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * The Exception class to indicate that an AcmeElement specified could not be
 * found.
 */
public class ElementNotFoundException extends NestingException {

    /** Message indicating that an element cannot be found */
    public static final String NO_ELEMENT = "Unable to find the AcmeElement specified";
    /** Message indicating that a design cannot be found */
    public static final String NO_DESIGN = "Unable to find the AcmeDesign specified";
    /** Message indicating that a system cannot be found */
    public static final String NO_SYS = "Unable to find the AcmeSystem specified";
    /** Message indicating that a representation (though not technically an
     *  AcmeElement) cannot be found */
    public static final String NO_REP = "Unable to find the AcmeRepresentation specified";
    /** Message indicating that a component cannot be found */
    public static final String NO_COMP = "Unable to find AcmeComponent specified";
    /** Message indicating that a connector cannot be found */
    public static final String NO_CONN = "Unable to find AcmeConnector specified";
    /** Message indicating that a port cannot be found */
    public static final String NO_PORT = "Unable to find AcmePort specified";
    /** Message indicating that a role cannot be found */
    public static final String NO_ROLE = "Unable to find AcmeRole specified";
    /** Message indicating that an attachment cannot be found */
    public static final String NO_ATTACH = "Unable to find AcmeAttachment specified";
    /** Message indicating that a property cannot be found */
    public static final String NO_PROP = "Unable to find AcmeProperty specified";

    private String _elementName = null;

    /**
     * Constructs an exception indicating that an element cannot be found,
     * with no triggering Throwable.
     * @param msg  The message to indicate the reason for exception
     */
    public ElementNotFoundException(String msg) {
        this(msg, null, null);
    }

    /**
     * Constructs an exception indicating that an element cannot be found,
     * using the name of the element supplied, without a source Throwable.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the AcmeElement
     */
    public ElementNotFoundException(String msg, String name) {
        this(msg, name, null);
    }

    /**
     * Constructs an exception indicating that an element cannot be found.
     * @param msg  The message to indicate the reason for exception
     * @param nestedException  Triggering throwable, if any
     */
    public ElementNotFoundException(String msg, Throwable nestedEx) {
        this(msg, null, nestedEx);
    }

    /**
     * Constructs an exception indicating that an element cannot be found,
     * using the name of the element supplied.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the AcmeElement
     * @param nestedException  Triggering throwable, if any
     */
    public ElementNotFoundException(String msg, String name, Throwable nestedEx) {
        super(msg + (name == null ? "!" : ": " + name), nestedEx);
    }

}
