/*
 * Created on Jul 23, 2003
 */
package AcmeStudio.remote.exception;


/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * Indicates that a supplied name is illegal.
 */
public class SetupDesignException extends NestingException {

    /** Message indicating that the operation to open an editor failed */
    public static final String OPEN_FAILED = "Open editor failed!";
    /** Message indicating that the named project could not be determined */
    public static final String PROJECT_NOT_FOUND = "Cannot determine the project";
    /** Message indicating that the named file could not be found */
    public static final String FILE_NOT_FOUND = "Cannot find file in project";

    private String _name = null;

    /**
     * Constructs an exception indicating that setup failed, on the item
     * name supplied, without a source Throwable.
     * @param msg  The message to indicate the reason for exception
     */
    public SetupDesignException(String msg) {
        super(msg);
    }

    /**
     * Constructs an exception indicating that setup failed, on the item
     * name supplied, without a source Throwable.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the offending object, e.g., project or file name
     */
    public SetupDesignException(String msg, String name) {
        this(msg, name, null);
    }

    /**
     * Constructs an exception indicating that setup failed.
     * @param msg  The message to indicate the reason for exception
     * @param nestedEx  The nested exception
     */
    public SetupDesignException(String msg, Throwable nestedEx) {
        this(msg, null, nestedEx);
    }


    /**
     * Constructs an exception indicating that setup failed, on the item
     * name supplied.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the offending object, e.g., project or file name
     * @param nestedEx  The nested exception
     */
    public SetupDesignException(String msg, String name, Throwable nestedEx) {
        super(msg + (name == null ? "!" : ": " + name), nestedEx);
    }
}
