/*
 * Created on Jul 31, 2003
 */
package AcmeStudio.remote.exception;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * The Exception class to indicate that a type specified could not be found.
 */
public class TypeNotFoundException extends NestingException {

    /** Message indicating that a family cannot be found */
    public static final String NO_FAMILY = "Unable to find the AcmeFamily specified";
    /** Message indicating that an element type cannot be found */
    public static final String NO_ELEMENT_TYPE = "Unable to find the AcmeElementType specified";
    /** Message indicating that a component type cannot be found */
    public static final String NO_COMP_TYPE = "Unable to find the AcmeComponentType specified";
    /** Message indicating that a connector type cannot be found */
    public static final String NO_CONN_TYPE = "Unable to find the AcmeConnectorType specified";
    /** Message indicating that a port type cannot be found */
    public static final String NO_PORT_TYPE = "Unable to find the AcmePortType specified";
    /** Message indicating that a role type cannot be found */
    public static final String NO_ROLE_TYPE = "Unable to find the AcmeRoleType specified";
    /** Message indicating that a property type cannot be found */
    public static final String NO_PROP_TYPE = "Unable to find the AcmePropertyType specified";

    private String _typeName = null;

    /**
     * Constructs an exception indicating that an element cannot be found,
     * with no triggering Throwable.
     * @param msg  The message to indicate the reason for exception
     */
    public TypeNotFoundException(String msg) {
        this(msg, null, null);
    }

    /**
     * Constructs an exception indicating that an element cannot be found,
     * using the name of the element supplied, without a source Throwable.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the type, e.g., component or connector type
     */
    public TypeNotFoundException(String msg, String name) {
        this(msg, name, null);
    }

    /**
     * Constructs an exception indicating that an element cannot be found.
     * @param msg  The message to indicate the reason for exception
     * @param nestedException  Triggering throwable, if any
     */
    public TypeNotFoundException(String msg, Throwable nestedEx) {
        this(msg, null, nestedEx);
    }

    /**
     * Constructs an exception indicating that an element cannot be found,
     * using the name of the element supplied.
     * @param msg  The message to indicate the reason for exception
     * @param name  The name of the type, e.g., component or connector type
     * @param nestedException  Triggering throwable, if any
     */
    public TypeNotFoundException(String msg, String name, Throwable nestedEx) {
        super(msg + (name == null ? "!" : ": " + name), nestedEx);
    }

}
