/*
 * Created on Jul 5, 2005
 *  
 */
package AcmeStudio.remote.jms;

import java.io.IOException;
import java.io.PipedWriter;
import java.util.StringTokenizer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.w3c.dom.*;
import org.w3c.dom.CharacterData;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.CreateElementCommand;
import AcmeStudio.model.DeleteElementCommand;
import AcmeStudio.model.PropertyCommand;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.remote.exception.SetupDesignException;
import AcmeStudio.remote.exception.TypeNotFoundException;
import AcmeStudio.remote.impl.ASInfo;
import AcmeStudio.ui.editors.diagram.actions.SetElementConstraintCommand;
import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.styles.Location;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;
import acmeLib.properties.*;
/**
 * @author Bradley Schmerl Created Jul 5, 2005 Translate mutation events that
 *                have xAcme elements in them to changes on the archtiecture in
 *                AcmeStudio
 */
public class XAcmeToAcmeTranslator {

	/** Info about the current state of AcmeStudio * */
	private ASInfo				_info;
	/** The writer to put information out on */
	private final PipedWriter	_pipe;
	/** An object for synchronization * */
	private Object				_asynchInProgress	= new Object ();
	/** The display to use to run updates against */
	private Display				_display;
	/** The page of the editor */
	protected IWorkbenchPage	_page;

	/**
	 * Constructs the translator and sets the up information
	 * 
	 * @param pipe
	 *                    the pipe to pipe debugging strings to
	 */
	public XAcmeToAcmeTranslator(PipedWriter pipe) {
		_pipe = pipe;
		_display = Display.getDefault ();
		// Set up the page
		_display.asyncExec (new Runnable () {

			public void run() {
				_page = AcmeStudioPlugin.getActiveWindow ().getActivePage ();
			}
		});
	}

	/**
	 * Processes the mutation element
	 * 
	 * @param xAcmeDoc
	 *                    The document that contains the mutation
	 * @throws acmeLib.core.exceptions.AcmeIllegalNameException
	 */
	public void processXAcmeMutation(org.w3c.dom.Document xAcmeDoc) throws acmeLib.core.exceptions.AcmeIllegalNameException {
		// First, check to see if this is the beginning or end of a change
		Element documentElement = xAcmeDoc.getDocumentElement ();
		processXAcmeMutation (documentElement);
	}

	/**
	 * Processes the mutation
	 * 
	 * @param documentElement
	 *                    The element that contains the mutation
	 */
	public void processXAcmeMutation(Element documentElement) {
		NodeList elementsChanging = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.STARTEDCHANGES_TAG);
		NodeList elementsChanged = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.FINISHEDCHANGES_TAG);
		// Get all the objects to be deleted in the global scope
		NodeList objectsCreated = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.CREATED_TAG);
		// Get all the objects to be deleted in the global scope
		NodeList objectsDeleted = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.DELETED_TAG);
		// Get all the objects to be changed in the global scope
		NodeList objectsChanged = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.CHANGED_TAG);
		// Get all the objects to be attached in the global scope
		NodeList objectsAttached = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.ATTACHED_TAG);
		// Get all the objects to be detached in the global scope
		NodeList objectsDetached = XMLImporter.getElementsAtThisLevel (documentElement, XAcmeConstants.DETACHED_TAG);
		try {
			if (elementsChanging.getLength () != 0) {
				//				AcmeStudioChangeManager.globalChangeManager.beginChanges ();
			}
			onCreated (objectsCreated);
			onDeleted (objectsDeleted);
			onAttached (objectsAttached);
			//			onDetached (objectsDetached);
			//			onChanged (objectsChanged);
			if (elementsChanged.getLength () != 0) {
				//				AcmeStudioChangeManager.globalChangeManager.endChanges ();
			}
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * Processes attachment mutations
	 * 
	 * @param objectsAttached
	 *                    the nodes containing attachment mutations
	 */
	private void onAttached(NodeList objectsAttached) {
		int i, j;
		for (i = 0; i < objectsAttached.getLength (); i++) {
			NodeList newAttachments = XMLImporter.getElementsAtThisLevel ((Element )objectsAttached.item (i),
					XAcmeConstants.ATTACHEDCONNECTOR_TAG);
			for (j = 0; j < newAttachments.getLength (); j++) {
				Element newAttachment = (Element )newAttachments.item (j);
				try {
					onAttachmentCreated (newAttachment);
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
		}
	}

	/**
	 * Creates an attachment and changes model through AcmeStudio
	 * 
	 * @param attachment
	 *                    The element describing the new attachment
	 */
	private void onAttachmentCreated(Element attachment) {
		try {
			String port = getRef (attachment, "portName");
			String role = getRef (attachment, "roleName");
			pipe ("attaching " + port + ":" + role + XAcmeConstants.EOL);
			AcmePort portACME = (AcmePort )getNamedElement (port);
			AcmeRole roleACME = (AcmeRole )getNamedElement (role);
			AcmeAttachment attach = new AcmeAttachment (portACME, roleACME);
			attach.setParent (_info.system ());
			createAttachment (_info.system (), attach);
			pipe ("=> Added '" + attach.getTypeSignature () + "' to '" + _info.system ().getTypeSignature () + "'"
					+ XAcmeConstants.EOL);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * Creates the new attachment
	 * 
	 * @param system
	 *                    The system to attach to
	 * @param attach
	 *                    The new attachment
	 */
	private void createAttachment(final AcmeSystem system, final AcmeAttachment attach) {
		_display.asyncExec (new Runnable () {

			public void run() {
				synchronized (_asynchInProgress) {
					system.addAttachment (attach);
					AcmeStudioChangeManager.globalChangeManager.onAddAttachment (system, attach, this);
				}
			}
		});
	}

	/*
	 * Process the objects to be created
	 */
	private void onCreated(NodeList objectsCreated) {
		int i, j;
		for (i = 0; i < objectsCreated.getLength (); i++) {
			NodeList contextNodesXML = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i),
					XAcmeConstants.CONTEXT_TAG);
			String contextName = null;
			if (contextNodesXML.getLength () == 1) {
				Element contextXML = (Element )contextNodesXML.item (0);
				contextName = contextXML.getAttribute ("href");
			}
			NodeList newSystems = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i),
					XAcmeConstants.NEWSYSTEM_TAG);
			for (j = 0; j < newSystems.getLength (); j++) {
				Element newSystem = (Element )newSystems.item (j);
				onSystemCreated (newSystem);
			}
			NodeList newComponents = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i),
					XAcmeConstants.NEWCOMPONENT_TAG);
			for (j = 0; j < newComponents.getLength (); j++) {
				Element newComponent = (Element )newComponents.item (j);
				try {
					AcmeSystem system = _info.system ();
					if (contextName != null && !_info.system ().getName ().equals (contextName)) {
						Object newSystem = _info.design ().lookupQualifiedName (
								_info.system ().getQualifiedName () + "." + contextName);
						if (newSystem instanceof AcmeSystem) {
							system = (AcmeSystem )newSystem;
						}
					}
					onComponentCreated (newComponent, system);
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			NodeList newConnectors = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i),
					XAcmeConstants.NEWCONNECTOR_TAG);
			for (j = 0; j < newConnectors.getLength (); j++) {
				Element newConnector = (Element )newConnectors.item (j);
				try {
					AcmeSystem system = _info.system ();
					if (contextName != null && !_info.system ().getName ().equals (contextName)) {
						Object newSystem = _info.design ().lookupQualifiedName (
								_info.system ().getQualifiedName () + "." + contextName);
						if (newSystem instanceof AcmeSystem) {
							system = (AcmeSystem )newSystem;
						}
					}
					onConnectorCreated (newConnector, system);
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			NodeList newProperties = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i),
					XAcmeConstants.NEWPROPERTY_TAG);
			for (j = 0; j < newProperties.getLength (); j++) {
				Element newProperty = (Element )newProperties.item (j);
				try {
					AcmeElement contextElement = getNamedElement (contextName);
					if (contextElement != null) {
						onPropertyCreated (newProperty, contextElement);
					}
					else {
						pipe ("X> Unable to find element " + contextName + " to give a property to");
					}
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			NodeList newPorts = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i), XAcmeConstants.NEWPORT_TAG);
			for (j = 0; j < newPorts.getLength (); j++) {
				Element newPort = (Element )newPorts.item (j);
				if (contextName == null) {
					pipe ("X> Got a createPort without a component");
					return;
				}
				try {
					AcmeElement contextElement = getNamedElement (contextName);
					if (contextElement instanceof AcmeComponent)
						onPortCreated (newPort, (AcmeComponent )contextElement);
					else
						pipe ("X> Got createPort, but " + contextName + " is not a component!");
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			NodeList newRoles = XMLImporter.getElementsAtThisLevel ((Element )objectsCreated.item (i), XAcmeConstants.NEWROLE_TAG);
			for (j = 0; j < newRoles.getLength (); j++) {
				Element newRole = (Element )newRoles.item (j);
				if (contextName == null) {
					pipe ("X> Got a createRole without a connector");
					return;
				}
				try {
					AcmeElement contextElement = getNamedElement (contextName);
					if (contextElement instanceof AcmeConnector)
						onRoleCreated (newRole, (AcmeConnector )contextElement);
					else
						pipe ("X> Got a createRole, but " + contextName + " is not a connector!");
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			//			NodeList newBindings = XMLImporter.getElementsAtThisLevel
			// ((Element )objectsCreated.item (i), NEWBINDING_TAG);
			//			for (j = 0; j < newBindings.getLength (); j++) {
			//				Element newBinding = (Element )newBindings.item (j);
			//				try {
			//					onBindingCreated (newBinding);
			//				}
			//				catch (Exception e) {
			//					e.printStackTrace ();
			//				}
			//			}
			//			NodeList newGauges = XMLImporter.getElementsAtThisLevel ((Element
			// )objectsCreated.item (i), NEWGAUGE_TAG);
			//			for (j = 0; j < newGauges.getLength (); j++) {
			//				Element newGauge = (Element )newGauges.item (j);
			//				try {
			//					onGaugeCreated (newGauge);
			//				}
			//				catch (Exception e) {
			//					e.printStackTrace ();
			//				}
			//			}
		}
	}

	/**
	 * Returns the element corresponding to the name
	 * 
	 * @param name
	 *                    the name to look for
	 * @return the corresponding AcmeElement
	 */
	private AcmeElement getNamedElement(String name) {
		name = XAcmeTools.stripHash (name);
		StringTokenizer tokenizer = new StringTokenizer (name, ".");
		String firstToken = tokenizer.nextToken ();
		if (firstToken.equals (_info.system ().getName ())) {
			return (AcmeElement )_info.design ().lookupQualifiedName (name);
		}
		else {
			AcmeElement acmeElement = (AcmeElement )_info.design ().lookupQualifiedName (_info.system ().getName () + "." + name);
			return acmeElement;
		}
	}

	/*
	 * Process the objects to be deleted
	 */
	private void onDeleted(NodeList objectsDeleted) {
		int i, j;
		for (i = 0; i < objectsDeleted.getLength (); i++) {
			NodeList deletedComponents = XMLImporter.getElementsAtThisLevel ((Element )objectsDeleted.item (i),
					XAcmeConstants.DELETEDCOMPONENT_TAG);
			if (deletedComponents.getLength () > 0) {
				Element deletedComponent = (Element )objectsDeleted.item (i);
				try {
					onComponentDeleted (deletedComponent);
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			NodeList deletedConnectors = XMLImporter.getElementsAtThisLevel ((Element )objectsDeleted.item (i),
					XAcmeConstants.DELETEDCONNECTOR_TAG);
			if (deletedConnectors.getLength () > 0) {
				Element deletedConnector = (Element )objectsDeleted.item (i);
				try {
					onConnectorDeleted (deletedConnector);
				}
				catch (Exception e) {
					e.printStackTrace ();
				}
			}
			//			NodeList deletedPorts = XMLImporter.getElementsAtThisLevel
			// ((Element )objectsDeleted.item (i),
			// XAcmeConstants.DELETEDPORT_TAG);
			//			if (deletedPorts.getLength () > 0) {
			//				Element deletedPort = (Element )objectsDeleted.item (i);
			//				try {
			//					onPortDeleted (deletedPort);
			//				}
			//				catch (Exception e) {
			//					e.printStackTrace ();
			//				}
			//			}
			//			NodeList deletedRoles = XMLImporter.getElementsAtThisLevel
			// ((Element )objectsDeleted.item (i),
			// XAcmeConstants.DELETEDROLE_TAG);
			//			if (deletedRoles.getLength () > 0) {
			//				Element deletedRole = (Element )objectsDeleted.item (i);
			//				try {
			//					onRoleDeleted (deletedRole);
			//				}
			//				catch (Exception e) {
			//					e.printStackTrace ();
			//				}
			//			}
		}
	}

	/**
	 * @param system
	 */
	private void onSystemCreated(Element system) {
		String sysName = getName (system);
		IProject proj = null;
		proj = ASContext.current ().getActiveProject ();
		if (proj == null) {
			pipe ("X> Unable to find currently active project!");
		}
		String filename = sysName + ".acme";
		final IFile file = (IFile )proj.findMember (filename);
		if (file == null) {
			pipe ("X> Unabel to find file " + filename + "!");
			return;
		}
		_info = new ASInfo ();
		String absName = null;
		try {
			absName = file.getLocation ().toFile ().getCanonicalPath ();
		}
		catch (IOException e) {
			pipe ("X> " + e.toString ());
			e.printStackTrace ();
			absName = file.getLocation ().toString ();
		}
		_info.setFileName (absName);
		pipe ("==> Opening editor on file " + absName + "");
		_display.syncExec (new Runnable () {

			public void run() {
				synchronized (_asynchInProgress) {
					try {
						_info.setEditPart (_page.openEditor (new FileEditorInput (file),
								"AcmeStudio.ui.editors.MultiPageAcmeEditor", OpenStrategy.activateOnOpen ()));
						_info.setSystemView ((SystemView )_info.mPAE ().getDiagramEditor ().getRootEditPart ().getFigure ());
						AcmeOpenedDesignRecord rec = AcmeDesign.getDesignRecord (_info.fileName ());
						if (rec != null) {
							AcmeDesign design = rec.getResolvedDesign ();
							if (design != null) {
								_info.setDesign (design);
								AcmeSystem sys = (AcmeSystem )design.getSystems ().getNextItem ();
								_info.setSystem (sys);
							}
						}
					}
					catch (PartInitException e) {
						throw new RuntimeException (
								new SetupDesignException (SetupDesignException.OPEN_FAILED, file.toString (), e));
					}
				}
			}
		});
	}

	/*
	 * Process the component to be created
	 */
	private void onComponentCreated(Element comp, AcmeSystem system) {
		try {
			String compName = getName (comp);
			String typeName = getType (comp);
			pipe ("=>creating component: " + compName + " ofType " + typeName + XAcmeConstants.EOL);
			AcmeComponentType compTypeACME = (AcmeComponentType )system.lookupName (typeName, true);
			String uName = Helper.getUniqueName (system, compName);
			AcmeComponent compACME = new AcmeComponent (uName, compTypeACME);
			compACME.addInstantiatedType (compTypeACME);
			compACME.addDeclaredType (compTypeACME);
			compACME.instantiateElements ();
			createElement (system, compACME);
			_pipe.write ("=> Added " + compACME.getTypeSignature () + " to " + _info.system ().getTypeSignature ()
					+ XAcmeConstants.EOL);
			NodeList interfacesDOM = XMLImporter.getElementsAtThisLevel (comp, "interfaceInstance");
			int i;
			for (i = 0; i < interfacesDOM.getLength (); i++) {
				Element interfaceDOM = (Element )interfacesDOM.item (i);
				onPortCreated (interfaceDOM, compACME);
			}
			NodeList propertiesDOM = XMLImporter.getElementsAtThisLevel (comp, "property");
			for (i = 0; i < propertiesDOM.getLength (); i++) {
				Element propertyDOM = (Element )propertiesDOM.item (i);
				onPropertyCreated (propertyDOM, compACME);
			}
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/*
	 * Process the component to be deleted
	 */
	private void onComponentDeleted(Element compXML) {
		try {
			String componentName = getRef (compXML, XAcmeConstants.DELETEDCOMPONENT_TAG);
			pipe ("deleting component: " + componentName + XAcmeConstants.EOL);
			AcmeComponent compACME = (AcmeComponent )_info.design ().lookupQualifiedName (componentName);
			deleteElement (_info.system (), compACME);
			pipe ("=> Removed " + compACME.getTypeSignature () + XAcmeConstants.EOL);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * @param system
	 * @param compACME
	 */
	private void deleteElement(final AcmeElement parent, final AcmeElement child) {
		_display.asyncExec (new Runnable () {

			public void run() {
				synchronized (_asynchInProgress) {
					DeleteElementCommand delete = new DeleteElementCommand (parent, child, this);
					String label = "Delete " + child.getTypeSignature ();
					delete.setLabel (label);
					delete.setDebugLabel (label);
					delete.execute (_info.design ());
				}
			}
		});
	}

	/*
	 * Process the connector to be created
	 */
	private void onConnectorCreated(Element conn, AcmeSystem system) {
		try {
			String connName = getName (conn);
			String typeName = getType (conn);
			pipe ("creating connector: " + connName + " ofType " + typeName + XAcmeConstants.EOL);
			AcmeConnectorType connTypeACME = (AcmeConnectorType )system.lookupName (typeName, true);
			String uName = Helper.getUniqueName (system, connName);
			AcmeConnector connACME = new AcmeConnector (uName, connTypeACME);
			connACME.addInstantiatedType (connTypeACME);
			connACME.addDeclaredType (connTypeACME);
			connACME.instantiateElements ();
			createElement (system, connACME);
			NodeList interfacesDOM = XMLImporter.getElementsAtThisLevel (conn, "interfaceInstance");
			int i;
			for (i = 0; i < interfacesDOM.getLength (); i++) {
				Element interfaceDOM = (Element )interfacesDOM.item (i);
				onRoleCreated (interfaceDOM, connACME);
			}
			NodeList propertiesDOM = XMLImporter.getElementsAtThisLevel (conn, "property");
			for (i = 0; i < propertiesDOM.getLength (); i++) {
				Element propertyDOM = (Element )propertiesDOM.item (i);
				onPropertyCreated (propertyDOM, connACME);
			}
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * @param deletedConnector
	 */
	private void onConnectorDeleted(Element connXML) {
		try {
			String connName = getRef (connXML, XAcmeConstants.DELETEDCONNECTOR_TAG);
			pipe ("deleting component: " + connName + XAcmeConstants.EOL);
			AcmeConnector connACME = (AcmeConnector )_info.design ().lookupQualifiedName (connName);
			deleteElement (_info.system (), connACME);
			pipe ("=> Removed " + connACME.getTypeSignature () + XAcmeConstants.EOL);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/*
	 * Process the property to be created
	 */
	private void onPropertyCreated(Element prop, AcmeElement hostACME) {
		try {
			String propName = getName (prop);
			String propValue = "";
			AcmeProperty tempPropACME = null;
			String propType = "";
			if (hostACME == null) {
				hostACME = (AcmeElement )_info.design ().lookupQualifiedName (getParentFullName (prop));
			}
			NodeList typeDOM = XMLImporter.getElementsAtThisLevel (prop, "propertyType");
			if (typeDOM.getLength () > 0) {
				propType = getPropertyType ((Element )typeDOM.item (0));
			}
			NodeList valuesDOM = XMLImporter.getElementsAtThisLevel (prop, "value");
			if (valuesDOM.getLength () > 0) {
				propValue = getPropertyValue ((Element )valuesDOM.item (0));
				tempPropACME = determinePropVal (_info.system (), propType, propValue);
			}
			AcmeProperty propertyACME = new AcmeProperty (propName, tempPropACME.getType (), tempPropACME.getValue ());
			pipe ("=> creating property: " + propName + "@" + hostACME.getQualifiedName () + " ofType " + propType + " ofValue "
					+ propValue);
			createProperty (hostACME, propertyACME);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * @param hostACME
	 * @param propertyACME
	 */
	private void createProperty(final AcmeElement hostACME, final AcmeProperty propertyACME) {
		_display.asyncExec (new Runnable () {

			public void run() {
				synchronized (_asynchInProgress) {
					if (isVisProp (propertyACME)
							&& AcmeStudioPlugin.getDefault ().getPreferenceStore ().getBoolean (
									GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
						SystemView sysView = _info.systemView ();
						Point location = sysView.getElementLocation (hostACME);
						Dimension size = Helper.getSize (hostACME);
						if (propertyACME.getName ().equals ("vis-x")) {
							location.x = (int )((AcmeFloat )propertyACME.getValue ()).getValue ();
						}
						else if (propertyACME.getName ().equals ("vis-y")) {
							location.y = (int )((AcmeFloat )propertyACME.getValue ()).getValue ();
						}
						else if (propertyACME.getName ().equals ("vis-width")) {
							size.height = (int )((AcmeFloat )propertyACME.getValue ()).getValue ();
						}
						else if (propertyACME.getName ().equals ("vis-height")) {
							size.width = (int )((AcmeFloat )propertyACME.getValue ()).getValue ();
						}
						EditPart editPart = _info.mPAE ().getDiagramEditor ().getEditPart (hostACME);
						if (editPart != null) {
							SetElementConstraintCommand command = new SetElementConstraintCommand ((GraphicalEditPart )editPart);
							command.setSystemView (sysView);
							command.setElement (hostACME);
							command.setLocation (new Rectangle (location, size));
							command.setLabel ("Change location/size (remote)");
							command.setDebugLabel ("Change location or size (done by remote control)");
							command.execute ();
						}
					}
					else {
						PropertyCommand propCmd = new PropertyCommand (PropertyCommand.ADD_PROP, hostACME, propertyACME, this);
						String label = "Add property " + propertyACME.getQualifiedName ();
						propCmd.setLabel (label);
						propCmd.setDebugLabel (label);
						propCmd.execute (_info.design ());
					}
				}
			}

			private boolean isVisProp(AcmeProperty propertyACME) {
				return propertyACME.getName ().startsWith ("vis-");
			}
		});
	}

	/*
	 * Process the port to be created
	 */
	private void onPortCreated(Element port, AcmeComponent componentACME) {
		try {
			String portName = getName (port);
			String portType = getType (port);
			pipe ("creating port: " + portName + "@" + componentACME.getQualifiedName () + " ofType " + portType
					+ XAcmeConstants.EOL);
			AcmePortType portT = (AcmePortType )_info.system ().lookupName (portType, true);
			String uName = Helper.getUniqueName (componentACME, portName);
			AcmePort portACME = new AcmePort (uName, portT);
			portACME.addInstantiatedType (portT);
			portACME.addDeclaredType (portT);
			portACME.instantiateElements ();
			createElement (componentACME, portACME);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/*
	 * Process the port to be created
	 */
	private void onRoleCreated(Element roleDOM, AcmeConnector connACME) {
		try {
			String roleName = getName (roleDOM);
			String roleType = getType (roleDOM);
			pipe ("creating role: " + roleName + "@" + connACME.getQualifiedName () + " ofType " + roleType + XAcmeConstants.EOL);
			AcmePortType roleTypeACME = (AcmePortType )_info.system ().lookupName (roleType, true);
			String uName = Helper.getUniqueName (connACME, roleName);
			AcmePort portACME = new AcmePort (uName, roleTypeACME);
			portACME.addInstantiatedType (roleTypeACME);
			portACME.addDeclaredType (roleTypeACME);
			portACME.instantiateElements ();
			createElement (connACME, portACME);
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
	}

	/**
	 * @param system
	 * @param comp
	 */
	private void createElement(final AcmeElement parent, final AcmeElement child) {
		_display.asyncExec (new Runnable () {

			public void run() {
				synchronized (_asynchInProgress) {
					try {
						SystemView sysView = _info.systemView ();
						Point location = new Point (0, 0);
						CreateElementCommand create = new CreateElementCommand (parent, child, location, this);
						Figure parentVis = (Figure )_info.mPAE ().getDiagramEditor ().getFigure (parent);
						if (parentVis instanceof AcmeVisElement) {
							create.setParentVis ((AcmeVisElement )parentVis);
						}
						create.setSystemView (sysView);
						Dimension size = Helper.getSize (child);
						if (size != null) {
							sysView.updateElement (child, new Rectangle (location, size));
							sysView.computeChildLocations ();
							Location newL = null;
							if ((newL = sysView.getCalculatedLocation (child)) != null) {
								location = newL.getRectangle ().getLocation ();
							}
						}
						String label = "Create " + child.getTypeSignature ();
						create.setLabel (label);
						create.setDebugLabel (label);
						create.execute (_info.design ());
					}
					finally {
						_asynchInProgress.notifyAll ();
					}
				}
			}
		});
	}

	private String getFullName(Element elem) {
		String name = "";
		try {
			name = XAcmeTools.removeColon (elem.getAttribute ("id"));
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return name;
	}

	private String getName(Element elem) {
		String name = "";
		try {
			name = XAcmeTools.decomposeName (elem.getAttribute ("id"));
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return name;
	}

	private String getType(Element elem) {
		String typeName = "";
		try {
			NodeList typesDOM = XMLImporter.getElementsAtThisLevel (elem, "type");
			if (typesDOM.getLength () > 0) {
				Element typeDOM = (Element )typesDOM.item (0);
				typeName = XAcmeTools.stripHash (typeDOM.getAttribute ("xlink:href"));
				if (typeName == null || typeName.equals ("")) typeName = XAcmeTools.stripHash (typeDOM.getAttribute ("href"));
			}
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return typeName;
	}

	private String getParentFullName(Element elem) {
		String name = "";
		try {
			name = XAcmeTools.getParentFullName (elem.getAttribute ("id"));
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return name;
	}

	/**
	 * Takes a property type name and property value, determines the actual
	 * AcmePropertyType and the AcmePropertyValue, and returns those in an
	 * anonymous AcmePropety.
	 * 
	 * @param sys
	 *                    AcmeSystem in which to look up the property type
	 * @param type
	 *                    The type of the property
	 * @param val
	 *                    The value of the property
	 * @return AcmeProperty The anonymous AcmeProperty that contains the
	 *               resolved AcmePropertyType and AcmePropertyValue
	 * @throws NumberFormatException
	 * @throws TypeNotFoundException
	 */
	protected AcmeProperty determinePropVal(final AcmeSystem sys, String type, String val) throws NumberFormatException,
			TypeNotFoundException {
		// find the type referred to, and determine info value type
		AcmePropertyType propT = (AcmePropertyType )sys.lookupName (type, true);
		AcmePropertyValue propV = null;
		if (propT == null) { // determine a basic type
			if (type.equalsIgnoreCase ("int")) {
				propT = ConstantTypes.INT_TYPE;
				propV = new AcmeInt ((IntType )propT, Integer.parseInt (val));
			}
			else if (type.equalsIgnoreCase ("float")) {
				propT = ConstantTypes.FLOAT_TYPE;
				propV = new AcmeFloat ((FloatType )propT, Float.parseFloat (val));
			}
			else if (type.equalsIgnoreCase ("boolean")) {
				propT = ConstantTypes.BOOLEAN_TYPE;
				propV = new AcmeBoolean ((BooleanType )propT, Boolean.getBoolean (val));
			}
			else if (type.equalsIgnoreCase ("string")) {
				propT = ConstantTypes.STRING_TYPE;
				propV = new AcmeString ((StringType )propT, val);
			}
			else if (type.equalsIgnoreCase ("enum")) {
				propT = ConstantTypes.EMPTY_ENUM_TYPE;
				propV = new AcmeEnum ((EnumType )propT, val);
			}
			else if (type.equalsIgnoreCase ("set")) {
				propT = ConstantTypes.ANY_SET_TYPE;
			}
			else if (type.equalsIgnoreCase ("sequence")) {
				propT = ConstantTypes.ANY_SEQUENCE_TYPE;
			}
			else if (type.equalsIgnoreCase ("record")) {
				propT = ConstantTypes.ANY_RECORD_TYPE;
			}
			else if (type.equalsIgnoreCase ("any")) {
				propT = ConstantTypes.EXPLICIT_ANY_TYPE;
				propV = new AcmeString (val);
			}
			else {
				propT = ConstantTypes.UNSPECIFIED_TYPE;
				propV = new AcmeString (val);
			}
		}
		if (propT == null) { // shouldn't happen
			pipe ("X> Unable to determine AcmePropertyType for " + type + "!");
		}
		AcmeProperty prop = new AcmeProperty ("_TEMP_", propT, propV);
		return prop;
	}

	/**
	 * @param string
	 * @throws IOException
	 */
	private void pipe(String string) {
		try {
			_pipe.write (string + XAcmeConstants.EOL);
		}
		catch (IOException e) {
			// Ignore
		}
	}

	private String getPropertyValue(Element valueDOM) {
		String value = "";
		Element actualValueDOM = XMLImporter.findFirstElement (valueDOM);
		if (actualValueDOM.getNodeName ().equals ("propertyPrimitiveValue")) {
			Element primValueDOM = XMLImporter.findFirstElement (actualValueDOM);
			Node firstNode = actualValueDOM.getFirstChild ();
			CharacterData cvalue = (CharacterData )actualValueDOM.getFirstChild ();
			value = cvalue.getData ();
		}
		return value;
	}

	/**
	 * Translates a DOM propertyType node to string
	 */
	private String getPropertyType(Element propertyTypeDOM) {
		String type = "";
		if (!propertyTypeDOM.getNodeName ().equals (XAcmeConstants.PROPERTYTYPE_TAG)) {
			System.err.println ("property type tag does not match");
			return type;
		}
		else {
			NamedNodeMap attributes = propertyTypeDOM.getAttributes ();
			Attr name = (Attr )attributes.getNamedItem ("name");
			NodeList details = propertyTypeDOM.getChildNodes ();
			String acmeName = null;
			if (name != null) {
				acmeName = XAcmeTools.decomposeName (name.getValue ());
			}
			Element typeDetailDOM = XMLImporter.findFirstElement (propertyTypeDOM);
			if (typeDetailDOM.getNodeName ().equals ("propertyPrimitiveType")) {
				NamedNodeMap pType = typeDetailDOM.getAttributes ();
				Attr typeAttr = (Attr )pType.getNamedItem ("type");
				if (typeAttr == null) // for backward bug compatability
					typeAttr = (Attr )pType.getNamedItem ("name");
				type = typeAttr.getValue ().toLowerCase ();
			}
		}
		return type;
	}

	private String getRef(Element elem, String tag) {
		String ref = "";
		try {
			NodeList refsDOM = XMLImporter.getElementsAtThisLevel (elem, tag);
			if (refsDOM.getLength () > 0) {
				Element refDOM = (Element )refsDOM.item (0);
				//				NodeList instancesDOM = refDOM.getChildNodes();
				// //XMLImporter.getElementsAtThisLevel (refDOM, "instanceOf");
				//				if (instancesDOM.getLength () > 0) {
				//					Element instanceDOM = (Element )instancesDOM.item (0);
				ref = XAcmeTools.stripHash (refDOM.getAttribute ("xlink:href"));
				if (ref == null || ref.equals ("")) ref = XAcmeTools.stripHash (refDOM.getAttribute ("href"));
				//				}
			}
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return ref;
	}

	/**
	 * @param info
	 */
	public void setASInfo(ASInfo info) {
		_info = info;
	}
}
