/*
 * Created on Jul 5, 2005
 *  
 */
package AcmeStudio.remote.jms;

import org.w3c.dom.Element;
/**
 * @author Bradley Schmerl Created Jul 5, 2005
 */
public class XAcmeTools {

	private String getFullName(Element elem) {
		String name = "";
		try {
			name = removeColon (elem.getAttribute ("id"));
		}
		catch (Exception e) {
			e.printStackTrace ();
		}
		return name;
	}

	/**
	 * return the Acme identifier of a fully qualified unique xml identifier
	 * Assumed xml naming convention is: something.something.acmeName::number
	 * this function returns something.something.acmeName
	 * 
	 * @param complexName
	 *                    String
	 * @return the Acme name
	 */
	public static String removeColon(String complexName) {
		// Returns the Acme name of a fully qualified Acme name
		String afterLastDot = complexName;
		String upToColon;
		if (afterLastDot.indexOf (':') == -1) {
			upToColon = afterLastDot;
		}
		else {
			upToColon = afterLastDot.substring (0, afterLastDot.indexOf (':'));
		}
		return upToColon;
	}

	/**
	 * return the Acme identifier of a fully qualified unique xml identifier
	 * Assumed xml naming convention is: something.something.acmeName::number
	 * this function returns acmeName
	 * 
	 * @param complexName
	 *                    String
	 * @return the Acme name
	 */
	public static String decomposeName(String complexName) {
		// Returns the Acme name of a fully qualified Acme name
		String afterLastDot;
		if (complexName.lastIndexOf ('.') == -1) {
			afterLastDot = complexName;
		}
		else {
			afterLastDot = complexName.substring (complexName.lastIndexOf ('.') + 1, complexName.length ());
		}
		String upToColon;
		if (afterLastDot.indexOf (':') == -1) {
			upToColon = afterLastDot;
		}
		else {
			upToColon = afterLastDot.substring (0, afterLastDot.indexOf (':'));
		}
		return upToColon;
	}

	/**
	 * return the Acme identifier of a fully qualified unique xml identifier
	 * Assumed xml naming convention is: something.something.acmeName::number
	 * this function returns acmeName
	 * 
	 * @param complexName
	 *                    String
	 * @return the Acme name
	 */
	public static String getParentName(String complexName) {
		// Returns the Acme name of a fully qualified Acme name
		String upToLastDot, result;
		upToLastDot = complexName.substring (0, complexName.lastIndexOf ('.'));
		result = upToLastDot.substring (upToLastDot.lastIndexOf ('.') + 1, upToLastDot.length ());
		return result;
	}

	/**
	 * return the Acme identifier of a fully qualified unique xml identifier
	 * Assumed xml naming convention is: something.something.acmeName::number
	 * this function returns acmeName
	 * 
	 * @param complexName
	 *                    String
	 * @return the Acme name
	 */
	public static String getParentFullName(String complexName) {
		// Returns the Acme name of a fully qualified Acme name
		String upToLastDot, result;
		upToLastDot = complexName.substring (0, complexName.lastIndexOf ('.'));
		result = upToLastDot.substring (0, upToLastDot.length ());
		return result;
	}

	/**
	 * Strips the hash from the beginning of a name
	 * 
	 * @param nameWithHash
	 *                    String
	 * @return nameWithoutHash
	 */
	public static String stripHash(String nameWithHash) {
		String nameWithoutHash = nameWithHash;
		if (nameWithHash.indexOf ('#') != -1) {
			nameWithoutHash = nameWithHash.substring (nameWithHash.indexOf ('#') + 1, nameWithHash.length ());
		}
		return nameWithoutHash;
	}
}
