/*
 * Created on Jul 17, 2003
 */
package AcmeStudio.remote.rmi;

import java.rmi.MarshalledObject;
import java.rmi.RemoteException;
import java.rmi.activation.Activatable;
import java.rmi.activation.ActivationID;

import AcmeStudio.remote.IAcmeStudio;
import AcmeStudio.remote.IAcmeStudioDesign;
import AcmeStudio.remote.impl.RemoteControlServer;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * The class provides remote control access to AcmeStudio.  It serves remote
 * access requests from clients via RMI, and the methods correspond to
 * architecture change operations.
 */
public class RemoteControlActivatable
    extends Activatable
    implements IAcmeStudio {

    private RemoteControlServer _server = null;

    /**
     * Creates and exports a new RemoteControlServer Activatable object of the
     * given id, and setup data object, using anonymous port.
     * @param id  the ActivationID that identifies the object to initialize.
     * @param data  the setup data object, as a MarshalledObject.
     * @throws java.rmi.RemoteException  if failed to export object
     */
    public RemoteControlActivatable(ActivationID id, MarshalledObject data)
        throws RemoteException {
        super(id, 0);
        
        _server = RemoteControlServer.getInstance();
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioRemoteControl#serverReady()
     */
    public boolean serverReady() throws RemoteException {
        return _server.serverReady();
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioRemoteControl#setupSystem(String, String)
     */
    public IAcmeStudioDesign setupDesign (String fileName, String projName)
        throws RemoteException {

        return _server.setupDesign(fileName, projName);
    }

    }
