/*
 * Created on Jul 17, 2003
 */
package AcmeStudio.remote.rmi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.List;

import AcmeStudio.remote.IAcmeStudioDesign;
import AcmeStudio.remote.exception.ElementNotFoundException;
import AcmeStudio.remote.exception.TypeNotFoundException;
import AcmeStudio.remote.impl.RemoteDesign;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * The class provides remote control access to AcmeStudio.  It serves remote
 * access requests from clients via RMI, and the methods correspond to
 * architecture change operations.
 */
public class RemoteDesignUnicast
    extends UnicastRemoteObject
    implements IAcmeStudioDesign {

    /** The remote AcmeDesign handled by this object */
    private RemoteDesign _design = null;

    /**
     * Creates and exports a new RemoteControlServer UnicastRemoteServer object
     * using anonymous port.
     * @throws java.rmi.RemoteException  if failed to export object
     */
    public RemoteDesignUnicast(RemoteDesign design)
        throws RemoteException {

        super();
        _design = design;
    }


    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#close()
     */
    public void close ()
        throws RemoteException {

        _design.close();
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getSystem()
     */
    public String getSystem()
        throws RemoteException {

        return _design.getSystem();
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getSystem(java.lang.String)
     */
    public String getSystem(String repName)
        throws RemoteException {

        return _design.getSystem(repName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getPorts(java.lang.String)
     */
    public List getPorts(String compName)
        throws RemoteException, ElementNotFoundException {

        return _design.getPorts(compName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getPortsOfType(java.lang.String, java.lang.String)
     */
    public List getPortsOfType(String compName, String type)
        throws RemoteException {

        return _design.getPortsOfType(compName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getRoles(java.lang.String)
     */
    public List getRoles(String connName)
        throws RemoteException, ElementNotFoundException {

        return _design.getRoles(connName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#getRolesOfType(java.lang.String, java.lang.String)
     */
    public List getRolesOfType(String connName, String type)
        throws RemoteException {

        return _design.getRolesOfType(connName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#isPortAvailable(java.lang.String)
     */
    public boolean isPortAvailable(String portName)
        throws RemoteException, ElementNotFoundException {

        return _design.isPortAvailable(portName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#isRoleAvailable(java.lang.String)
     */
    public boolean isRoleAvailable(String roleName)
        throws RemoteException, ElementNotFoundException {

        return _design.isRoleAvailable(roleName);
    }


    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addComponent(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addComponent(String sysName, String compName, String type)
        throws RemoteException {

        return _design.addComponent(sysName, compName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addConnector(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addConnector(String sysName, String connName, String type)
        throws RemoteException {

        return _design.addConnector(sysName, connName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addPort(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addPort(String compName, String portName, String type)
        throws RemoteException {

        return _design.addPort(compName, portName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addRole(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addRole(String connName, String roleName, String type)
        throws RemoteException {

        return _design.addRole(connName, roleName, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addAttachment(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addAttachment(String sysName, String portName, String roleName)
        throws RemoteException {

        return _design.addAttachment(sysName, portName, roleName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addRepresentation(java.lang.String, java.lang.String, java.lang.String)
     */
    public String addRepresentation (String element, String name, String fam)
        throws RemoteException {

        return _design.addRepresentation(element, name, fam);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addBinding(java.lang.String, java.lang.String)
     */
    public String addBinding(String extPort, String intPort)
        throws RemoteException {

        return _design.addBinding(extPort, intPort);
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.remote.IAcmeStudioDesign#addGauge(java.lang.String, java.lang.String, java.lang.String)
	 */
	public String addGauge(String elemName, String gaugeName, String type)
			throws RemoteException, ElementNotFoundException,
			TypeNotFoundException {

        return _design.addGauge(elemName, gaugeName, type);
	}


    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeComponent(java.lang.String, java.lang.String)
     */
    public void removeComponent(String sysName, String compName)
        throws RemoteException {

        _design.removeComponent(sysName, compName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeConnector(java.lang.String, java.lang.String)
     */
    public void removeConnector(String sysName, String connName)
        throws RemoteException {

        _design.removeConnector(sysName, connName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removePort(java.lang.String)
     */
    public void removePort(String portName) throws RemoteException {
        _design.removePort(portName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeRole(java.lang.String)
     */
    public void removeRole(String roleName) throws RemoteException {
        _design.removeRole(roleName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeAttachment(java.lang.String, java.lang.String)
     */
    public void removeAttachment(String portName, String roleName)
        throws RemoteException {
        _design.removeAttachment(portName, roleName);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeRepresentation(java.lang.String)
     */
    public void removeRepresentation(String name)
        throws RemoteException {

        _design.removeRepresentation(name);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeBinding(java.lang.String, java.lang.String)
     */
    public void removeBinding(String extPort, String intPort)
        throws RemoteException {

        _design.removeBinding(extPort, intPort);
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.remote.IAcmeStudioDesign#removeGauge(java.lang.String)
	 */
	public void removeGauge(String gaugeName) throws RemoteException,
			ElementNotFoundException {

		_design.removeGauge(gaugeName);
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.remote.IAcmeStudioDesign#attachGauge(java.lang.String, java.lang.String, java.lang.String)
	 */
	public String attachGauge(String propName, String gaugeName, String mapping)
			throws RemoteException, ElementNotFoundException {

        return _design.attachGauge(propName, gaugeName, mapping);
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.remote.IAcmeStudioDesign#detachGauge(java.lang.String, java.lang.String)
	 */
	public String detachGauge(String propName, String gaugeName)
			throws RemoteException, ElementNotFoundException {

        return _design.detachGauge(propName, gaugeName);
	}


    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#setPropety(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public String setProperty(String elementName, String propName,String type, String val)
        throws RemoteException {

        return _design.setProperty(elementName, propName, type, val);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#addArmaniRule(java.lang.String, java.lang.String)
     */
    public String addArmaniRule(String elementName, String rule)
        throws RemoteException {

        return _design.addArmaniRule(elementName, rule);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeProperty(java.lang.String, java.lang.String)
     */
    public void removeProperty(String name, String prop) throws RemoteException {

        _design.removeProperty(name, prop);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#removeArmaniRule(java.lang.String, java.lang.String)
     */
    public void removeArmaniRule(String elementName, String rule)
        throws RemoteException {

        _design.removeArmaniRule(elementName, rule);
    }


    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#assignFamily(java.lang.String, java.lang.String)
     */
    public void assignFamily(String sys, String fam)
        throws RemoteException {

        _design.assignFamily(sys, fam);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#assignType(java.lang.String, java.lang.String)
     */
    public void assignType(String target, String type)
        throws RemoteException {

        _design.assignType(target, type);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#unassignFamily(java.lang.String, java.lang.String)
     */
    public void unassignFamily(String sys, String fam)
        throws RemoteException {

        _design.unassignFamily(sys, fam);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.remote.IAcmeStudioDesign#unassignType(java.lang.String, java.lang.String)
     */
    public void unassignType(String target, String type)
        throws RemoteException {

        _design.unassignType(target, type);
    }

}
