/*
 * Created on Jul 17, 2003
 */
package AcmeStudio.remote.test;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import AcmeStudio.remote.IAcmeStudio;
import AcmeStudio.remote.IAcmeStudioDesign;
import AcmeStudio.remote.RemoteControlSecurityManager;
import AcmeStudio.remote.test.statement.Statement;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 * 
 * TODO Comments for RemoteControlTest
 */
public class RemoteControlTest {

    /** The remote interface to the remote object */
    private static IAcmeStudio _ias = null;

    public static void simpleTest () throws RemoteException {
        int id = 0;

        System.out.print("Invoking remote call to setup design...");
        IAcmeStudioDesign iasd = _ias.setupDesign("System1.acme", "Test");
        System.out.println("done! => Design ID: " + id);

        System.out.print("Getting System name...");
        String sys = iasd.getSystem();
        System.out.println("done! -> " + sys);

        System.out.print("Adding a component 1...");
        String comp1 = iasd.addComponent(sys, "whoMe", "Filter");
        System.out.println("done! -> " + comp1);

        System.out.print("Adding a component 2...");
        String comp2 = iasd.addComponent(sys, "whoYou", "Filter");
        System.out.println("done! -> " + comp2);

        System.out.print("Adding a port to component 1...");
        String portC1 = iasd.addPort(comp1, "p", "outputT");
        System.out.println("done! -> " + portC1);

        System.out.print("Adding a port to component 2...");
        String portC2 = iasd.addPort(comp2, "p", "inputT");
        System.out.println("done! -> " + portC2);

        System.out.print("Adding a connector...");
        String conn = iasd.addConnector(sys, "conn", "Pipe");
        System.out.println("done! -> " + conn);

        System.out.print("Adding a source to connector...");
        String src = iasd.addRole(conn, "src", "sourceT");
        System.out.println("done! -> " + src);

        System.out.print("Adding a sink to connector...");
        String snk = iasd.addRole(conn, "snk", "sinkT");
        System.out.println("done! -> " + snk);

        System.out.print("Attaching component 1's port to connector source...");
        String attach1 = iasd.addAttachment(sys, portC1, src);
        System.out.println("done! -> " + attach1);

        System.out.print("Attaching component 2's port to connector sink...");
        String attach2 = iasd.addAttachment(sys, portC2, snk);
        System.out.println("done! -> " + attach2);
    }

    public static void main(String[] args) {
        // Set a security manager so that the client can 
        // download the remote server object's stub
        System.setSecurityManager(new RemoteControlSecurityManager());

        String server = null;
        String object = null;
        String codebase = null;
        int i = 0;
        ResourceBundle bundle = ResourceBundle.getBundle("AcmeStudio.remote.acmestudioremoteplugin");
        if (args.length < i+1 || args[i] == null) {
            try {
                server = bundle.getString("RemoteControlView.text.hostname");
            } catch (MissingResourceException e) {
                server = bundle.getString("Plugin.host");
            }
        } else {
            server = args[i];
        }
        ++i;
        if (args.length < i+1 || args[i] == null) {
            try {
                codebase = bundle.getString("RemoteControlView.text.codebase");
            } catch (MissingResourceException e) {
                System.err.println("Unable to determine local codebase, please supply as argument 2!");
                System.exit(-1);
            }
        } else {
            codebase = args[i];
        }
        ++i;
        if (args.length < i+1 || args[i] == null) {
            object = AcmeStudio.remote.rmi.RemoteControlUnicast.class.getName();
        } else {
            object = args[i];
        }
        System.setProperty("java.rmi.server.codebase", codebase);
        System.out.println("** java.rmi.server.codebase set to " + System.getProperty("java.rmi.server.codebase") + "'");
        System.setProperty("java.rmi.server.useCodebaseOnly", "true");

        try {
            String location = "rmi://" + server + "/" + object;

            _ias = (IAcmeStudio) Naming.lookup(location);
            System.out.println("Got a remote reference to " + location);

            // Insertion.begin
//            simpleTest();
//            System.exit(0);
            // Insertion.end

            String initName = "test/init.script";
            File initFile = new File(initName);
            if (initFile.exists()) {
                System.out.println("Loading init script...");
                Statement.parse("run \"" + initName + "\"").execute(_ias);
            }
            System.out.println();
            System.out.println("Please begin issuing commands, enter \"help()\" for help.");
            System.out.println();

            BufferedReader sin = new BufferedReader(new InputStreamReader(System.in));
            while (true) {
                System.out.print("=> ");
                String input = sin.readLine().trim();
                if (input.equalsIgnoreCase("quit") || input.equalsIgnoreCase("exit")) {
                    // quit time!
                    break;
                } else if (! input.equals("")){
                    if (! Statement.parse(input).execute(_ias)) {
                        System.err.println("** Execution of '" + input + "' failed!");
                    }
                }
            }
        } catch (Exception e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }

        System.out.println();
        System.out.println("RemoteControlTest completes!");
        System.out.println();
    }
}
